#include "console_output.h"

#include <string>
#include <utility>
#include <ncurses.h>


console_output::console_output(std::ostream &cout): out{cout} {}

/* Attributes
    black        30         40
    red          31         41
    green        32         42
    yellow       33         43
    blue         34         44
    magenta      35         45
    cyan         36         46
    white        37         47
    reset             0  (everything back to normal)
    bold/bright       1  (often a brighter shade of the same colour)
    underline         4
    inverse           7  (swap foreground and background colours)
    bold/bright off  21
    underline off    24
    inverse off      27

   Format:
    \033[X;Ym
    X Y are numbers from above
*/

std::string console_output::get_code(const int attr) {
        if (attr < 255)
                return "\033[0m";

        std::string result = "\033[";

        if (attr & A_STANDOUT)
                result += "1;";

        if (attr & A_UNDERLINE)
                result += "4;";

        if (attr & COLOR_PAIR(COLOR_BLACK))
                result += "30;";

        if (attr & COLOR_PAIR(COLOR_RED))
                result += "31;";

        if (attr & COLOR_PAIR(COLOR_GREEN))
                result += "32;";

        if (attr & COLOR_PAIR(COLOR_YELLOW))
                result += "33;";

        if (attr & COLOR_PAIR(COLOR_BLUE))
                result += "34;";

        if (attr & COLOR_PAIR(COLOR_MAGENTA))
                result += "35;";

        if (attr & COLOR_PAIR(COLOR_CYAN))
                result += "36;";

        if (attr & COLOR_PAIR(COLOR_WHITE))
                result += "37;";

        result[result.length() - 1] = 'm';
        return result;
}

void console_output::render() {
        for (std::size_t idx = 0; idx < contents.size(); ++idx) {
                if (idx % DISPLAY_WIDTH == 0 && idx)
                        out << std::endl;

                out << get_code(contents[idx]) +
                    (char)(contents[idx] ? contents[idx] : ' ');
        }
}

void console_output::clear() {
        contents.clear();
        contents.reserve(DISPLAY_BUFFER_SIZE);
}

void console_output::print_char(const position &pos, const char ch,
                                const int attr) {
        if (pos.x >= DISPLAY_WIDTH || pos.y >= DISPLAY_HEIGHT)
                return;

        contents[pos.y * DISPLAY_WIDTH + pos.x] = attr | ch;
}

void console_output::print_str(const position &pos, const std::string &str,
                               const int attr) {
        if (pos.x >= DISPLAY_WIDTH || pos.y >= DISPLAY_HEIGHT)
                return;

        int head = pos.y * DISPLAY_WIDTH + pos.x;

        for (std::size_t i = 0; i < str.length(); ++i)
                contents[i + head] = attr | str[i];
}
